/*
 * Sonar, entreprise quality control tool.
 * Copyright (C) 2007-2008 Hortis-GRC SA
 * mailto:be_agile HAT hortis DOT ch
 *
 * Sonar is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * Sonar is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with Sonar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02
 */
package org.sonar.plugins.checkstyle;

import org.apache.commons.io.FileUtils;
import org.apache.maven.model.Plugin;
import org.apache.maven.project.MavenProject;
import org.sonar.plugins.api.PluginContext;
import org.sonar.plugins.api.Languages;
import org.sonar.plugins.maven.AbstractMavenPluginHandler;
import org.sonar.plugins.maven.MavenUtils;
import org.sonar.plugins.rules.RulesPluginContext;

import java.io.File;
import java.io.IOException;

public class CheckstyleMavenPluginHandler extends AbstractMavenPluginHandler {
  private RulesPluginContext rulesPluginContext;
  private CheckstyleRulesRepository checkstyleRulesRepository;

  public CheckstyleMavenPluginHandler(PluginContext pluginContext, CheckstyleRulesRepository checkstyleRulesRepository) {
    this.rulesPluginContext = pluginContext.getRulesContext(CheckstylePlugin.KEY);
    this.checkstyleRulesRepository = checkstyleRulesRepository;
  }

  protected CheckstyleMavenPluginHandler(RulesPluginContext rulesPluginContext, CheckstyleRulesRepository checkstyleRulesRepository) {
    this.rulesPluginContext = rulesPluginContext;
    this.checkstyleRulesRepository = checkstyleRulesRepository;
  }

  public boolean shouldStopOnFailure() {
    return true;
  }

  public boolean shouldExecuteOn(MavenProject pom) {
    return Languages.JAVA.equals(MavenUtils.getLanguage(pom));
  }

  public String getGroupId() {
    return MavenUtils.APACHE_MOJO_GROUP_ID;
  }

  public String getArtifactId() {
    return "maven-checkstyle-plugin";
  }

  public String getVersion() {
    return "2.2";
  }

  public boolean isFixedVersion() {
    return true;
  }

  public String[] getGoals() {
    return new String[]{"checkstyle"};
  }

  public void configurePlugin(MavenProject pom, Plugin plugin) {
    unsetConfigParameter(plugin, "outputDirectory");
    unsetConfigParameter(plugin, "outputFile");
    setConfigParameter(plugin, "outputFileFormat", "xml");
    setConfigParameter(plugin, "consoleOutput", "false");
    setConfigParameter(plugin, "enableRSS", "false");
    setConfigParameter(plugin, "failsOnError", "false");
    setConfigParameter(plugin, "linkXRef", "false");

    File xmlFile = null;
    try {
      xmlFile = saveConfigXml(pom);
      setConfigParameter(plugin, "configLocation", xmlFile.getCanonicalPath());

    } catch (IOException e) {
      throw new RuntimeException("fail to save the checkstyle XML configuration", e);
    }

    addRuleExtensionsDependency(plugin);
  }

  private File saveConfigXml(MavenProject pom) throws IOException {
    File checkstyle = new File(getWorkingDirectory(pom), "checkstyle.xml");
    String xml = checkstyleRulesRepository.exportConfiguration(rulesPluginContext.getActiveProfile());
    FileUtils.writeStringToFile(checkstyle, xml);
    return checkstyle;
  }
}
